/*******************************************************************************
 * Copyright (c) 2000, 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
module dwt.dnd.HTMLTransfer;

import dwt.internal.gtk.OS;
import dwt.dnd.ByteArrayTransfer;
import dwt.dnd.TransferData;
import dwt.dnd.DND;
import dwt.dwthelper.utils;

static import tango.text.Util;

/**
 * The class <code>HTMLTransfer</code> provides a platform specific mechanism
 * for converting text in HTML format represented as a java <code>String</code>
 * to a platform specific representation of the data and vice versa.  See
 * <code>Transfer</code> for additional information.
 *
 * <p>An example of a java <code>String</code> containing HTML text is shown
 * below:</p>
 *
 * <code><pre>
 *     String htmlData = "<p>This is a paragraph of text.</p>";
 * </code></pre>
 */
public class HTMLTransfer : ByteArrayTransfer {

    private static HTMLTransfer _instance;
    private static const char[] TEXT_HTML = "text/html"; //$NON-NLS-1$
    private static const int TEXT_HTML_ID;
    private static const char[] TEXT_HTML2 = "TEXT/HTML"; //$NON-NLS-1$
    private static const int TEXT_HTML2_ID;

static this(){
    _instance = new HTMLTransfer();
    TEXT_HTML_ID = registerType(TEXT_HTML);
    TEXT_HTML2_ID = registerType(TEXT_HTML2);
}

private this() {}

/**
 * Returns the singleton instance of the HTMLTransfer class.
 *
 * @return the singleton instance of the HTMLTransfer class
 */
public static HTMLTransfer getInstance () {
    return _instance;
}

/**
 * This implementation of <code>javaToNative</code> converts HTML-formatted text
 * represented by a java <code>String</code> to a platform specific representation.
 * For additional information see <code>Transfer#javaToNative</code>.
 *
 * @param object a java <code>String</code> containing HTML text
 * @param transferData an empty <code>TransferData</code> object; this
 *  object will be filled in on return with the platform specific format of the data
 */
public void javaToNative (Object object, TransferData transferData){
    transferData.result = 0;
    if (!checkHTML(object) || !isSupportedType(transferData)) {
        DND.error(DND.ERROR_INVALID_DATA);
    }
    char[] string = (cast(ArrayWrapperString)object).array;
    char* pValue = cast(char*)OS.g_malloc(string.length);
    if (pValue is null) return;
    pValue[0 .. string.length ] = string;
    transferData.length = string.length;
    transferData.format = 8;
    transferData.pValue = pValue;
    transferData.result = 1;
}

/**
 * This implementation of <code>nativeToJava</code> converts a platform specific
 * representation of HTML text to a java <code>String</code>.
 * For additional information see <code>Transfer#nativeToJava</code>.
 *
 * @param transferData the platform specific representation of the data to be
 * been converted
 * @return a java <code>String</code> containing HTML text if the
 * conversion was successful; otherwise null
 */
public Object nativeToJava(TransferData transferData){
    if ( !isSupportedType(transferData) ||  transferData.pValue is null ) return null;
    /* Ensure byteCount is a multiple of 2 bytes */
    int size = (transferData.format * transferData.length / 8) / 2 * 2;
    if (size <= 0) return null;
    char[] chars = transferData.pValue[ 0 .. size ].dup;
    return new ArrayWrapperString( chars[ 0 .. tango.text.Util.locate( chars, '\0' ) ] );
}
protected int[] getTypeIds() {
    return [TEXT_HTML_ID, TEXT_HTML2_ID];
}

protected char[][] getTypeNames() {
    return [TEXT_HTML, TEXT_HTML2];
}

bool checkHTML(Object object) {
    if( object is null ) return false;
    auto arr = cast(ArrayWrapperString)object;
    if( arr is null ) return false;
    if( arr.array.length is 0 ) return false;
    return true;
}

protected bool validate(Object object) {
    return checkHTML(object);
}
}
